@@ -285,6 +285,8 @@ GRN_API int64_t grn_get_default_match_escalation_threshold(void);
 GRN_API grn_rc grn_set_default_match_escalation_threshold(int64_t threshold);
 GRN_API bool grn_is_back_trace_enable(void);
 GRN_API grn_rc grn_set_back_trace_enable(bool enable);
+GRN_API bool grn_is_reference_count_enable(void);
+GRN_API grn_rc grn_set_reference_count_enable(bool enable);
 
 GRN_API grn_rc grn_ctx_set_variable(grn_ctx *ctx,
                                     const char *name,
@@ -405,7 +407,7 @@ typedef uint32_t grn_column_flags;
 #define GRN_VOID                       (0x00)
 #define GRN_BULK                       (0x02)
 #define GRN_PTR                        (0x03)
-#define GRN_UVECTOR                    (0x04) /* vector of fixed size data especially grn_id */
+#define GRN_UVECTOR                    (0x04) /* vector of fixed size (uniform) data especially grn_id */
 #define GRN_PVECTOR                    (0x05) /* vector of grn_obj* */
 #define GRN_VECTOR                     (0x06) /* vector of arbitrary data */
 #define GRN_MSG                        (0x07)
@@ -842,6 +844,7 @@ struct _grn_search_optarg {
   float weight_float;
   grn_obj *query_options;
   grn_obj *max_element_intervals;
+  int *min_interval;
 };
 
 GRN_API grn_rc grn_obj_search(grn_ctx *ctx, grn_obj *obj, grn_obj *query,
@@ -1105,6 +1108,27 @@ GRN_API off_t grn_default_logger_get_rotate_threshold_size(void);
   }\
 } while (0)
 
+GRN_API grn_rc grn_slow_log_push(grn_ctx *ctx);
+GRN_API double grn_slow_log_pop(grn_ctx *ctx);
+GRN_API bool grn_slow_log_is_slow(grn_ctx *ctx, double elapsed_time);
+
+#define GRN_SLOW_LOG_PUSH(ctx, level) do {      \
+  if (grn_logger_pass(ctx, level)) {            \
+    grn_slow_log_push(ctx);                     \
+  }                                             \
+} while (false)
+
+#define GRN_SLOW_LOG_POP_BEGIN(ctx, level, elapsed_time) do { \
+  if (grn_logger_pass(ctx, level)) {                          \
+    double elapsed_time = grn_slow_log_pop(ctx);              \
+    if (grn_slow_log_is_slow(ctx, elapsed_time)) {
+
+
+#define GRN_SLOW_LOG_POP_END(ctx)               \
+    }                                           \
+  }                                             \
+} while (false)
+
 typedef struct _grn_query_logger grn_query_logger;
 
 struct _grn_query_logger {
