@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2009-2018  Brazil
+  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -246,6 +246,7 @@ typedef uint32_t grn_table_group_flags;
 #define GRN_TABLE_GROUP_CALC_MEAN       (0x01<<7)
 #define GRN_TABLE_GROUP_CALC_AGGREGATOR (0x01<<8)
 #define GRN_TABLE_GROUP_LIMIT           (0x01<<9)
+#define GRN_TABLE_GROUP_KEY_VECTOR_EXPANSION_POWER_SET (0x01<<10)
 
 typedef struct _grn_table_group_aggregator grn_table_group_aggregator;
 
@@ -295,12 +296,12 @@ grn_table_group_aggregator_get_expression(grn_ctx *ctx,
 
 struct _grn_table_group_result {
   grn_obj *table;
-  unsigned char key_begin;
-  unsigned char key_end;
+  uint8_t key_begin;
+  uint8_t key_end;
   int limit;
   grn_table_group_flags flags;
   grn_operator op;
-  unsigned int max_n_subrecs;
+  uint32_t max_n_subrecs;
   grn_obj *calc_target;
   grn_table_group_aggregator **aggregators;
   uint32_t n_aggregators;
@@ -314,7 +315,7 @@ grn_table_group_keys_parse(grn_ctx *ctx,
                            grn_obj *table,
                            const char *raw_sort_keys,
                            int32_t raw_sort_keys_size,
-                           int *n_keys);
+                           uint32_t *n_keys);
 GRN_API grn_rc grn_table_setoperation(grn_ctx *ctx, grn_obj *table1, grn_obj *table2,
                                       grn_obj *res, grn_operator op);
 GRN_API grn_rc grn_table_difference(grn_ctx *ctx, grn_obj *table1, grn_obj *table2,
@@ -337,13 +338,13 @@ GRN_API grn_obj *grn_table_select(grn_ctx *ctx,
 GRN_API grn_table_sort_key *
 grn_table_sort_key_from_str(grn_ctx *ctx,
                             const char *str, unsigned int str_size,
-                            grn_obj *table, unsigned int *nkeys);
+                            grn_obj *table, uint32_t *nkeys);
 GRN_API grn_table_sort_key *
 grn_table_sort_keys_parse(grn_ctx *ctx,
                           grn_obj *table,
                           const char *raw_sort_keys,
                           int32_t raw_sort_keys_size,
-                          int *n_keys);
+                          uint32_t *n_keys);
 GRN_API grn_rc grn_table_sort_key_close(grn_ctx *ctx,
                                         grn_table_sort_key *keys,
                                         uint32_t nkeys);
@@ -358,7 +359,7 @@ GRN_API grn_obj *grn_table_create_for_group(grn_ctx *ctx,
                                             const char *path,
                                             grn_obj *group_key,
                                             grn_obj *value_type,
-                                            unsigned int max_n_subrecs);
+                                            uint32_t max_n_subrecs);
 
 GRN_API unsigned int grn_table_get_subrecs(grn_ctx *ctx, grn_obj *table,
                                            grn_id id, grn_id *subrecbuf,
