@@ -1,5 +1,5 @@
-/* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/*
+  Copyright(C) 2011-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,8 +15,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_DAT_PREFIX_CURSOR_HPP_
-#define GRN_DAT_PREFIX_CURSOR_HPP_
+#pragma once
 
 #include "cursor.hpp"
 #include "vector.hpp"
@@ -35,7 +34,7 @@ class GRN_DAT_API PrefixCursor : public Cursor {
             const String &str,
             UInt32 min_length = 0,
             UInt32 offset = 0,
-            UInt32 limit = UINT32_MAX,
+            UInt32 limit = MAX_UINT32,
             UInt32 flags = 0);
 
   void close();
@@ -76,5 +75,3 @@ class GRN_DAT_API PrefixCursor : public Cursor {
 
 }  // namespace dat
 }  // namespace grn
-
-#endif  // GRN_DAT_PREFIX_CURSOR_HPP_
