@@ -1,6 +1,6 @@
 /*
   Copyright(C) 2013-2018  Brazil
-  Copyright(C) 2019-2021  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2019-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -127,10 +127,10 @@ mrb_grn_scan_info_put_index(mrb_state *mrb, mrb_value self)
   if (!mrb_nil_p(mrb_scorer_args_expr)) {
     scorer_args_expr = DATA_PTR(mrb_scorer_args_expr);
   }
-  grn_scan_info_put_index(ctx, si, index, sid, weight,
+  grn_scan_info_put_index(ctx, si, index, (uint32_t)sid, (float)weight,
                           scorer,
                           scorer_args_expr,
-                          scorer_args_expr_offset);
+                          (uint32_t)scorer_args_expr_offset);
   return self;
 }
 
@@ -173,7 +173,7 @@ mrb_grn_scan_info_set_weight_factor(mrb_state *mrb, mrb_value self)
   mrb_float factor;
   mrb_get_args(mrb, "f", &factor);
   scan_info *si = DATA_PTR(self);
-  grn_scan_info_set_weight_factor(si, factor);
+  grn_scan_info_set_weight_factor(si, (float)factor);
   return self;
 }
 
@@ -185,7 +185,7 @@ mrb_grn_scan_info_set_end(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "i", &end);
   si = DATA_PTR(self);
-  grn_scan_info_set_end(si, end);
+  grn_scan_info_set_end(si, (uint32_t)end);
   return self;
 }
 
@@ -213,7 +213,7 @@ mrb_grn_scan_info_set_flags(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "i", &flags);
   si = DATA_PTR(self);
-  grn_scan_info_set_flags(si, flags);
+  grn_scan_info_set_flags(si, (int)flags);
   return self;
 }
 
@@ -261,7 +261,7 @@ mrb_grn_scan_info_set_max_interval(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "i", &max_interval);
   si = DATA_PTR(self);
-  grn_scan_info_set_max_interval(si, max_interval);
+  grn_scan_info_set_max_interval(si, (int)max_interval);
   return self;
 }
 
@@ -284,7 +284,7 @@ mrb_grn_scan_info_set_additional_last_interval(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "i", &additional_last_interval);
   si = DATA_PTR(self);
-  grn_scan_info_set_additional_last_interval(si, additional_last_interval);
+  grn_scan_info_set_additional_last_interval(si, (int)additional_last_interval);
   return self;
 }
 
@@ -299,6 +299,76 @@ mrb_grn_scan_info_get_additional_last_interval(mrb_state *mrb, mrb_value self)
   return mrb_int_value(mrb, additional_last_interval);
 }
 
+static mrb_value
+mrb_grn_scan_info_set_max_element_intervals(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)(mrb->ud);
+  scan_info *si = DATA_PTR(self);
+
+  mrb_value mrb_max_element_intervals;
+  mrb_get_args(mrb, "A", &mrb_max_element_intervals);
+
+  if (mrb_nil_p(mrb_max_element_intervals)) {
+    grn_scan_info_set_max_element_intervals(ctx, si, NULL);
+    return self;
+  }
+
+  grn_obj max_element_intervals;
+  GRN_INT32_INIT(&max_element_intervals, GRN_OBJ_VECTOR);
+  mrb_int n = RARRAY_LEN(mrb_max_element_intervals);
+  mrb_int i;
+  for (i = 0; i < n; i++) {
+    mrb_value mrb_max_element_interval =
+      RARRAY_PTR(mrb_max_element_intervals)[i];
+    mrb_int max_element_interval = mrb_integer(mrb_max_element_interval);
+    GRN_INT32_PUT(ctx, &max_element_intervals, max_element_interval);
+  }
+  grn_scan_info_set_max_element_intervals(ctx, si, &max_element_intervals);
+  GRN_OBJ_FIN(ctx, &max_element_intervals);
+
+  return self;
+}
+
+static mrb_value
+mrb_grn_scan_info_get_max_element_intervals(mrb_state *mrb, mrb_value self)
+{
+  scan_info *si = DATA_PTR(self);
+  grn_obj *max_element_intervals = grn_scan_info_get_max_element_intervals(si);
+  size_t n = GRN_INT32_VECTOR_SIZE(max_element_intervals);
+  mrb_value mrb_max_element_intervals = mrb_ary_new_capa(mrb, n);
+  size_t i;
+  for (i = 0; i < n; i++) {
+    int32_t max_element_interval = GRN_INT32_VALUE_AT(max_element_intervals, i);
+    mrb_ary_push(mrb,
+                 mrb_max_element_intervals,
+                 mrb_int_value(mrb, max_element_interval));
+  }
+  return mrb_max_element_intervals;
+}
+
+static mrb_value
+mrb_grn_scan_info_set_min_interval(mrb_state *mrb, mrb_value self)
+{
+  scan_info *si;
+  mrb_int min_interval;
+
+  mrb_get_args(mrb, "i", &min_interval);
+  si = DATA_PTR(self);
+  grn_scan_info_set_min_interval(si, (int)min_interval);
+  return self;
+}
+
+static mrb_value
+mrb_grn_scan_info_get_min_interval(mrb_state *mrb, mrb_value self)
+{
+  scan_info *si;
+  int min_interval;
+
+  si = DATA_PTR(self);
+  min_interval = grn_scan_info_get_min_interval(si);
+  return mrb_int_value(mrb, min_interval);
+}
+
 static mrb_value
 mrb_grn_scan_info_set_similarity_threshold(mrb_state *mrb, mrb_value self)
 {
@@ -307,7 +377,7 @@ mrb_grn_scan_info_set_similarity_threshold(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "i", &similarity_threshold);
   si = DATA_PTR(self);
-  grn_scan_info_set_similarity_threshold(si, similarity_threshold);
+  grn_scan_info_set_similarity_threshold(si, (int)similarity_threshold);
   return self;
 }
 
@@ -330,7 +400,7 @@ mrb_grn_scan_info_set_quorum_threshold(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "i", &quorum_threshold);
   si = DATA_PTR(self);
-  grn_scan_info_set_quorum_threshold(si, quorum_threshold);
+  grn_scan_info_set_quorum_threshold(si, (int)quorum_threshold);
   return self;
 }
 
@@ -356,7 +426,7 @@ mrb_grn_scan_info_get_arg(mrb_state *mrb, mrb_value self)
   mrb_get_args(mrb, "i", &index);
 
   si = DATA_PTR(self);
-  arg = grn_scan_info_get_arg(ctx, si, index);
+  arg = grn_scan_info_get_arg(ctx, si, (int)index);
 
   return grn_mrb_value_from_grn_obj(mrb, arg);
 }
@@ -396,7 +466,7 @@ mrb_grn_scan_info_set_start_position(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "i", &start_position);
   si = DATA_PTR(self);
-  grn_scan_info_set_start_position(si, start_position);
+  grn_scan_info_set_start_position(si, (uint32_t)start_position);
   return self;
 }
 
@@ -635,7 +705,7 @@ mrb_grn_expression_array_reference(mrb_state *mrb, mrb_value self)
       mrb_int name_length;
 
       name = mrb_sym2name_len(mrb, mrb_symbol(mrb_key), &name_length);
-      var = grn_expr_get_var(ctx, expr, name, name_length);
+      var = grn_expr_get_var(ctx, expr, name, (unsigned int)name_length);
     }
     break;
   case MRB_TT_STRING :
@@ -643,7 +713,9 @@ mrb_grn_expression_array_reference(mrb_state *mrb, mrb_value self)
                            RSTRING_PTR(mrb_key), RSTRING_LEN(mrb_key));
     break;
   case MRB_TT_INTEGER :
-    var = grn_expr_get_var_by_offset(ctx, expr, mrb_integer(mrb_key));
+    var = grn_expr_get_var_by_offset(ctx,
+                                     expr,
+                                     (unsigned int)mrb_integer(mrb_key));
     break;
   default :
     mrb_raisef(mrb, E_ARGUMENT_ERROR,
@@ -749,11 +821,11 @@ mrb_grn_expression_parse(mrb_state *mrb, mrb_value self)
 
     mrb_flags = grn_mrb_options_get_lit(mrb, mrb_options, "flags");
     if (!mrb_nil_p(mrb_flags)) {
-      flags = mrb_integer(mrb_flags);
+      flags = (grn_expr_flags)mrb_integer(mrb_flags);
     }
   }
 
-  grn_expr_parse(ctx, expr, query, query_size, default_column,
+  grn_expr_parse(ctx, expr, query, (unsigned int)query_size, default_column,
                  default_mode, default_operator, flags);
   grn_mrb_ctx_check(mrb);
 
@@ -769,13 +841,15 @@ mrb_grn_expression_append_object(mrb_state *mrb, mrb_value self)
   grn_obj *object;
   mrb_value mrb_op;
   grn_operator op;
-  mrb_int n_args;
+  mrb_int mrb_n_args;
+  int n_args;
 
   expr = DATA_PTR(self);
-  mrb_get_args(mrb, "ooi", &mrb_object, &mrb_op, &n_args);
+  mrb_get_args(mrb, "ooi", &mrb_object, &mrb_op, &mrb_n_args);
 
   object = DATA_PTR(mrb_object);
   op = grn_mrb_value_to_operator(mrb, mrb_op);
+  n_args = (int)mrb_n_args;
   grn_expr_append_obj(ctx, expr, object, op, n_args);
   grn_mrb_ctx_check(mrb);
 
@@ -790,12 +864,14 @@ mrb_grn_expression_append_constant(mrb_state *mrb, mrb_value self)
   mrb_value mrb_constant;
   mrb_value mrb_op;
   grn_operator op;
-  mrb_int n_args;
+  mrb_int mrb_n_args;
+  int n_args;
 
   expr = DATA_PTR(self);
-  mrb_get_args(mrb, "ooi", &mrb_constant, &mrb_op, &n_args);
+  mrb_get_args(mrb, "ooi", &mrb_constant, &mrb_op, &mrb_n_args);
 
   op = grn_mrb_value_to_operator(mrb, mrb_op);
+  n_args = (int)mrb_n_args;
   switch (mrb_type(mrb_constant)) {
   case MRB_TT_FALSE :
     if (mrb_nil_p(mrb_constant)) {
@@ -823,16 +899,17 @@ mrb_grn_expression_append_constant(mrb_state *mrb, mrb_value self)
   case MRB_TT_INTEGER:
     grn_expr_append_const_int32(ctx,
                                 expr,
-                                mrb_integer(mrb_constant),
+                                (int32_t)mrb_integer(mrb_constant),
                                 op,
                                 n_args);
     break;
   case MRB_TT_SYMBOL :
     {
       const char *value;
-      mrb_int value_length;
+      mrb_int mrb_value_length;
 
-      value = mrb_sym2name_len(mrb, mrb_symbol(mrb_constant), &value_length);
+      value = mrb_sym2name_len(mrb, mrb_symbol(mrb_constant), &mrb_value_length);
+      unsigned int value_length = (unsigned int)mrb_value_length;
       grn_expr_append_const_str(ctx, expr, value, value_length, op, n_args);
     }
     break;
@@ -842,7 +919,7 @@ mrb_grn_expression_append_constant(mrb_state *mrb, mrb_value self)
       if (sizeof(mrb_float) == sizeof(float)) {
         grn_expr_append_const_float32(ctx,
                                       expr,
-                                      value,
+                                      (float)value,
                                       op,
                                       n_args);
       } else {
@@ -884,7 +961,7 @@ mrb_grn_expression_append_constant(mrb_state *mrb, mrb_value self)
         mrb_value mrb_table;
         grn_obj *domain;
 
-        id = mrb_integer(mrb_funcall(mrb, mrb_constant, "id", 0));
+        id = (grn_id)mrb_integer(mrb_funcall(mrb, mrb_constant, "id", 0));
         mrb_table = mrb_funcall(mrb, mrb_constant, "table", 0);
         domain = DATA_PTR(mrb_table);
         GRN_RECORD_INIT(&constant, 0, grn_obj_id(ctx, domain));
@@ -924,7 +1001,7 @@ mrb_grn_expression_append_operator(mrb_state *mrb, mrb_value self)
   mrb_get_args(mrb, "oi", &mrb_op, &n_args);
 
   op = grn_mrb_value_to_operator(mrb, mrb_op);
-  grn_expr_append_op(ctx, expr, op, n_args);
+  grn_expr_append_op(ctx, expr, op, (int)n_args);
   grn_mrb_ctx_check(mrb);
 
   return mrb_nil_value();
@@ -974,6 +1051,16 @@ grn_mrb_expr_init(grn_ctx *ctx)
   mrb_define_method(mrb, klass, "additional_last_interval=",
                     mrb_grn_scan_info_set_additional_last_interval,
                     MRB_ARGS_REQ(1));
+  mrb_define_method(mrb, klass, "max_element_intervals",
+                    mrb_grn_scan_info_get_max_element_intervals,
+                    MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "max_element_intervals=",
+                    mrb_grn_scan_info_set_max_element_intervals,
+                    MRB_ARGS_REQ(1));
+  mrb_define_method(mrb, klass, "min_interval",
+                    mrb_grn_scan_info_get_min_interval, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "min_interval=",
+                    mrb_grn_scan_info_set_min_interval, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "similarity_threshold",
                     mrb_grn_scan_info_get_similarity_threshold, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "similarity_threshold=",
@@ -1142,7 +1229,7 @@ grn_mrb_scan_info_build(grn_ctx *ctx,
     int start;
 
     mrb_si_data = RARRAY_PTR(mrb_sis)[i];
-    start = mrb_integer(mrb_funcall(mrb, mrb_si_data, "start", 0));
+    start = (int)mrb_integer(mrb_funcall(mrb, mrb_si_data, "start", 0));
     si = grn_scan_info_open(ctx, start);
     mrb_si = mrb_grn_scan_info_new(mrb, si);
     mrb_funcall(mrb, mrb_si, "apply", 1, mrb_si_data);
@@ -1174,7 +1261,7 @@ grn_mrb_expr_estimate_size(grn_ctx *ctx, grn_obj *expr, grn_obj *table)
   if (mrb->exc) {
     size = grn_table_size(ctx, table);
   } else {
-    size = mrb_integer(mrb_size);
+    size = (unsigned int)mrb_integer(mrb_size);
   }
 
   mrb_gc_arena_restore(mrb, arena_index);
